<?php

/**
 * @file
 * Provide views data for dblog.module.
 */


/**
 * Implements hook_views_data().
 */
function config_views_views_data() {
  /** @var \Drupal\Core\Config\TypedConfigManagerInterface $typed_config */
  $data = array();
  $typed_config = Drupal::service('config.typed');

  $all_definitions = $typed_config->getDefinitions();
  $all_definitions_keys = array_keys($all_definitions);

  // Small optimization: the config entities due to their prefixed nature always
  // end up in *. Other types might have this too so these are just candidates.
  $config_keys = preg_grep('/\*$/', $all_definitions_keys);
  $entity_types = \Drupal::entityTypeManager()->getDefinitions();
  // Registers an entity area handler per entity type.
  /**
   * @var string $entity_type
   * @var  Drupal\Core\Config\Entity\ConfigEntityTypeInterface $entity_info
   */
  foreach ($entity_types as $entity_type => $entity_info) {

    if (!$entity_info->isSubclassOf('Drupal\Core\Config\Entity\ConfigEntityInterface')) {
      continue;
    }

    // Exclude entity types, which cannot be rendered.
    if ($entity_info->hasListBuilderClass()) {
      $label = $entity_info->getLabel();
      $data['views']['entity_' . $entity_type] = array(
        'title' => t('Rendered entity - @label', array('@label' => $label)),
        'help' => t('Displays a rendered @label entity in an area.', array('@label' => $label)),
        'area' => array(
          'entity_type' => $entity_type,
          'id' => 'entity',
        ),
      );
    }

    if ($prefix = $entity_info->getConfigPrefix()) {
      $table_name = 'config_' . str_replace('.', '_', $prefix);
      $id = $entity_info->getKey('id');
      $data[$table_name]['table'] = array(
        'group' => t('Config'),
        'entity type' => $entity_type,
        'entity revision' => FALSE,
      );
      $data[$table_name]['table']['base'] = array(
        'title' => $entity_info->getLabel(),
        'field' => $id,
        'query_id' => 'views_entity_query',
      );

      $data[$table_name][$id]['field'] = array(
        'id' => 'standard',
        'title' => 'id',
      );
      $data[$table_name]['operation'] = array(
        'title' => t('Operations'),
        'help' => t('Operations'),
        'field' => array(
          'id' => 'config_entity_operations',
        ),
      );

      // We do not kno how many stars a given config prefix requires, for
      // example the schema for block is under block.block.*.*
      $keys = preg_grep('/^' . str_replace('.', '\.', $prefix) . '\.\*/', $config_keys);
      // There always will be just one. @TODO: add exception?
      $key = reset($keys);
      $definition = $all_definitions[$key];
      _views_views_config_process_schema($data[$table_name], $all_definitions, $all_definitions_keys, $definition);
    }
  }
//  print_r($data['config_user_role']);exit;
  return $data;
}

function _views_views_config_process_schema(array &$views_config_data, array $all_definitions, array $all_definitions_keys, array $definition, $prefix = '') {
  if (!isset($definition['type'])) {
    return;
  }
  $delimiter = $prefix ? '.' : '';
  $type = $definition['type'];
  // The type might be constant (for eg. type: views_display) or might be a
  // reference, for eg. type: views.field.[plugin_id] As it is impossible to
  // know just from the metadata what the exact type will end up in the
  // latter case, iterate over every possibility (views.field.boolean,
  // views.field.counter, etc).
  if (isset($all_definitions[$type])) {
    $new_definition = $all_definitions[$type];
    if (isset($new_definition['type'])) {
      _views_views_config_process_schema($views_config_data, $all_definitions, $all_definitions_keys, $new_definition, $prefix);
    }
    // If the new definition type has no type then the current one is a
    // primitve type (for eg. string). If there is a label and this is
    // neither a mapping or a sequence then add it to Views data.
    elseif (!isset($definition['mapping']) && !isset($definition['sequence']) && isset($definition['label'])) {
      _views_views_config_data_add($views_config_data, $definition, $prefix);
      // As there is no mapping or sequence, this prefix is done.
      return;
    }
  }
  else {
    // For field.[%parent.type].settings, find field.email.settings etc.
    // For this, create a PCRE mask replacing the [] with "any string" for eg.
    // field\..*\.settings
//    $left = substr($type, 0, strpos($type, '['));
//    $right = substr($type, strpos($type, ']') + 1);
//    $mask = str_replace('.', '\.', $left) . '.*' . str_replace('.', '\.', $right);
//    // Now find every possible type and recurse.
//    foreach (preg_grep('/^' . $mask . '/', $all_definitions_keys) as $key) {
//      _views_views_config_process_schema($views_config_data, $all_definitions, $all_definitions_keys, $all_definitions[$key], $prefix);
//    }
  }
  // Finished with the type-related recursions, now do the specific recursions.
  if (isset($definition['mapping'])) {
    foreach ($definition['mapping'] as $k => $new_definition) {
      if (is_array($new_definition)) {
        _views_views_config_process_schema($views_config_data, $all_definitions, $all_definitions_keys, $new_definition, "$prefix$delimiter$k");
      }
    }
  }
//  elseif (isset($definition['sequence'])) {
//    foreach ($definition['sequence'] as $new_definition) {
//      if (is_array($new_definition)) {
//        _views_views_config_process_schema($views_config_data, $all_definitions, $all_definitions_keys, $new_definition, "$prefix$delimiter*");
//      }
//    }
//  }

}

function _views_views_config_data_add(array &$views_config_data, array $definition, $prefix) {
  $views_config_data[$prefix] = array(
    'title' => $definition['label'],
    'help' => $prefix,
    'sort' => array(
      'id' => 'standard',
    ),
  );
  switch ($definition['type']) {
    case 'boolean':
      $views_config_data[$prefix]['filter']['id'] = 'string';
      $views_config_data[$prefix]['field']['id'] = 'standard';
      break;
    case 'integer':
      $views_config_data[$prefix]['field']['id'] = 'numeric';
      $views_config_data[$prefix]['filter']['id'] = 'numeric';
      $views_config_data[$prefix]['argument']['id'] = 'numeric';
      break;
    default:
      $views_config_data[$prefix]['field']['id'] = 'standard';
      $views_config_data[$prefix]['filter']['id'] = 'string';
      $views_config_data[$prefix]['argument']['id'] = 'string';
      break;
  }
}

/**
 * Implements hook_views_data().
 */
function config_views_views_data_sample() {
  $data = array();

  $data['config_views']['table']['group'] = t('Configuration');
  $data['config_views']['table']['wizard_id'] = 'config_entity';
  $data['config_views']['table']['base'] = array(
    'query_id' => 'config_entity_query',
    'field' => 'id',
    'title' => t('Configuration Entity'),
    'help' => t('Contains list of configuration entities.'),
  );

  $data['config_views']['id'] = array(
    'title' => t('Machine name'),
    'help' => t('Unique configuration entity ID.'),
    'field' => array(
      'id' => 'config_entity_id',
    ),
    'filter' => array(
      'id' => 'string',
    ),
    'argument' => array(
      'id' => 'string',
    ),
    'sort' => array(
      'id' => 'standard',
    ),
    'search' => array(
      'id' => 'standard',
    ),
  );

  $data['config_views']['label'] = array(
    'title' => t('Label'),
    'help' => t('Label'),
    'field' => array(
      'id' => 'config_entity_label',
    ),
    'filter' => array(
      'id' => 'string',
    ),
    'argument' => array(
      'id' => 'string',
    ),
    'sort' => array(
      'id' => 'standard',
    ),
    'search' => array(
      'id' => 'standard',
    ),
  );

  $data['config_views']['type'] = array(
    'title' => t('Type'),
    'help' => t('Configuration entity type.'),
    'field' => array(
      'id' => 'string',
    ),
    'filter' => array(
      'id' => 'string',
    ),
    'argument' => array(
      'id' => 'string',
    ),
    'sort' => array(
      'id' => 'standard',
    ),
    'search' => array(
      'id' => 'standard',
    ),
  );

  $data['config_views']['description'] = array(
    'title' => t('Description'),
    'help' => t('Description'),
    'field' => array(
      'id' => 'config_entity_description',
    ),
    'filter' => array(
      'id' => 'string',
    ),
    'argument' => array(
      'id' => 'string',
    ),
    'sort' => array(
      'id' => 'standard',
    ),
    'search' => array(
      'id' => 'standard',
    ),
  );

  $data['config_views']['operation'] = array(
    'title' => t('Operations'),
    'help' => t('Operations'),
    'field' => array(
      'id' => 'config_entity_operations',
    ),
  );


  return $data;

}

