<?php

/**
 * @file
 * Contains \Drupal\config_views\Plugin\views\query\ConfigEntityQuery.
 */

namespace Drupal\config_views\Plugin\views\query;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Database\Database;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\display\DisplayPluginBase;
use Drupal\Core\Database\DatabaseExceptionWrapper;
use Drupal\views\Plugin\views\join\JoinPluginBase;
use Drupal\views\Plugin\views\HandlerBase;
use Drupal\views\ResultRow;
use Drupal\views\ViewExecutable;
use Drupal\views\Views;
use Drupal\views\Plugin\views\query\QueryPluginBase;

/**
 * Views query plugin for an SQL query.
 *
 * @ingroup views_query_plugins
 *
 * @ViewsQueryNotInUse(
 *   id = "config_entity_query",
 *   title = @Translation("Configuration entity query"),
 *   help = @Translation("Query will be generated and run using the Drupal database API.")
 * )
 */
class ConfigEntityQuery extends QueryPluginBase {


  /**
   * An array of sections of the WHERE query. Each section is in itself
   * an array of pieces and a flag as to whether or not it should be AND
   * or OR.
   */
  public $where = array();

  /**
   * {@inheritdoc}
   */
  function execute(ViewExecutable $view) {
    $entity_type_id = 'node';
    $bundle = 'article';
    $config_entity_id = 'user_role';
    // Special case for block and field.
    // $list_builder->render($entity_type_id, $bundle);

    $list_builder =  \Drupal::entityTypeManager()->getListBuilder($config_entity_id);
    $list_builder->render();

    foreach ($list_builder->load() as $key => $item) {
      $results[$item->id()] = array(
          'id' => $item->id(),
          'label' => $item->label(),
          'operations' => array(
            '#type' => 'operations',
            '#links' => $list_builder->getOperations($item)
          ),
        );
    }

    foreach ($results as $result) {
      $view->result[] = new ResultRow($result);
    }
  }

  public function ensureTable($table, $relationship = NULL, JoinPluginBase $join = NULL) {
    return;
  }

  public function addField($table, $field, $alias = '', $params = array()) {
    return $alias;
  }

  public function addWhere($group, $field, $value = NULL, $operator = NULL) {
    // Ensure all variants of 0 are actually 0. Thus '', 0 and NULL are all
    // the default group.
    if (empty($group)) {
      $group = 0;
    }

    // Check for a group.
    if (!isset($this->where[$group])) {
      $this->setWhereGroup('AND', $group);
    }

    $this->where[$group]['conditions'][] = array(
      'field' => $field,
      'value' => $value,
      'operator' => $operator,
    );
  }

}
