<?php

/**
 * @file
 * Contains \Drupal\views\Plugin\views\query\Entity.
 */


namespace Drupal\config_views\Plugin\views\query;

use Drupal\views\Plugin\views\join\JoinPluginBase;
use Drupal\views\Plugin\views\query\QueryPluginBase;
use Drupal\views\Plugin\views\query\Sql;
use Drupal\views\ResultRow;
use Drupal\views\ViewExecutable;

/**
 * @todo.
 *
 * @ViewsQuery(
 *   id = "views_entity_query",
 *   title = @Translation("Entity Query"),
 *   help = @Translation("Query will be generated and run using the Drupal Entity Query API.")
 * )
 */
class EntityQuery extends Sql {

  protected $commands = array();

  protected $entityConditionGroups;

  public function ensureTable($table, $relationship = NULL, JoinPluginBase $join = NULL) {}
  public function addField($table, $field, $alias = '', $params = array()) {
    return $alias ? $alias : $field;
  }

  public function condition($group, $field, $value = NULL, $operator = NULL, $langcode = NULL) {
    $this->commands[$group][] = array(
      'method' => 'condition',
      'args' => array($field, $value, $operator, $langcode),
    );
  }

  public function exists($group, $field, $langcode = NULL) {
    $this->commands[$group][] = array(
      'method' => 'exists',
      'args' => array($field, $langcode),
    );
  }

  public function notExists($group, $field, $langcode = NULL) {
    $this->commands[$group][] = array(
      'method' => 'notExists',
      'args' => array($field, $langcode),
    );
  }

  function build(ViewExecutable $view) {
    // Store the view in the object to be able to use it later.
    $this->view = $view;

    $view->initPager();
  }
  /**
   * Executes the query and fills the associated view object with according
   * values.
   *
   * Values to set: $view->result, $view->total_rows, $view->execute_time,
   * $view->pager['current_page'].
   *
   * $view->result should contain an array of objects. The array must use a
   * numeric index starting at 0.
   *
   * @param view $view
   *   The view which is executed.
   */
  public function execute(ViewExecutable $view) {
    $this->group_operator = isset($this->group_operator) ? $this->group_operator : 'AND';
    $base_table = $this->view->storage->get('base_table');
    $data = \Drupal::service('views.views_data')->get($base_table);
    $entity_type = $data['table']['entity type'];
    $query = \Drupal::entityQuery($entity_type, $this->group_operator);
    $this->entityConditionGroups = array(
      $query,
    );
    $this->buildConditions();
    $ids = $query->execute();
    $results = \Drupal::entityTypeManager()->getStorage($entity_type)->loadMultiple($ids);
    foreach ($results as $result) {
      // @todo: toArray() doesn't return all properties.
      $entity = $result->toArray();
      $entity['type'] = $entity_type;
      $entity['entity'] = $result;
      $view->result[] = new ResultRow($entity);
    }
    $view->total_rows = count($view->result);
    $view->execute_time = 0;
  }

  protected function buildConditions() {
    foreach ($this->commands as $group => $grouped_commands) {
      $conditionGroup = $this->getConditionGroup($group);
      foreach ($grouped_commands as $command) {
        call_user_func_array(array($conditionGroup, $command['method']), $command['args']);
      }
    }
  }

  protected function getConditionGroup($group) {
    if (!isset($this->entityConditionGroups[$group])) {
      $query = $this->entityConditionGroups[0];
      $condition = isset($this->where[$group]) && $this->where[$group]['type'] == 'OR' ? $query->orConditionGroup() : $query->andConditionGroup();
      $query->condition($condition);
      $this->entityConditionGroups[$group] = $condition;
    }
    return $this->entityConditionGroups[$group];
  }
}
